/* This program is aimed to practice on operator overloading in C++*/

#ifndef RATIONAL_H
#define RATIONAL_H

#include <iostream>
#include <iomanip>
#include <conio.h>

class Rational{
     friend ostream &operator<<( ostream &, const Rational &);
     friend istream &operator>>( istream &, Rational &);

   public:
     Rational(int = 1, int = 1); //constructor with default values are 1
     Rational &reduce(Rational &);
     void printFP()const;
     Rational &operator +(Rational);//addition operator overloading
     Rational &operator -(Rational);//subtraction operator overloading
     Rational &operator *(Rational);//multiplication operator overloading
     Rational &operator /(Rational);//division operator overloading
     Rational &operator =(Rational);//equality operator overloading
     int operator <(Rational);      //relational operators overloading
     int operator >(Rational);

   private:
     int numerator;
     int denuminator;
     float result;

};

//friend function of output display
ostream &operator<<(ostream &output, const Rational & ration)
{
   output << ration.numerator <<" / " << ration.denuminator << "\n";

   return output;


}
//friend function of input
istream &operator>>(istream &input, Rational & ration)
{
   input >> setw(2) >> ration.numerator;
   input.ignore();
   input >> setw(2) >> ration.denuminator ;

   return input;

}
#endif

Rational::Rational(int a, int b)
{
  numerator = (a > 0 ? a: 1);
  denuminator = (b > 0 ? b: 1);
}

//This function simplfies rational number
Rational &Rational::reduce(Rational &r)
{
   int count, min;

   min = (r.numerator <= r.denuminator ? r.numerator : r.denuminator);

   for (count = 2; count <= min; count++){
        if(r.denuminator % count == 0 && r.numerator % count == 0){
            r.numerator /= count;
            r.denuminator /= count;
        }
   }

  return r;
}

// Display rational number in floating point form
void Rational::printFP()const
{
  cout << (float)numerator / denuminator << "\n";
}

Rational &Rational:: operator +(Rational r)
{
  int multiply = 0;
  Rational temp;

  if(this->denuminator % r.denuminator == 0)
    multiply = 1;

  if(r.denuminator % this->denuminator == 0)
    multiply = 2;

  switch(multiply){
     case 0: temp.denuminator = this->denuminator * r.denuminator;

             this->numerator *= r.denuminator;
             r.numerator *= this->denuminator;
             temp.numerator = this->numerator + r.numerator;

             this->numerator /= r.denuminator;
             r.numerator /= this->denuminator;
             break;

     case 1:  multiply = this->denuminator / r.denuminator;
              temp.denuminator = this->denuminator;

              r.numerator *= multiply;
              temp.numerator = this->numerator + r.numerator;

              r.numerator /= multiply;
              break;

     case 2:  multiply =  r.denuminator /this->denuminator ;
              temp.denuminator = r.denuminator;

              this->numerator *= multiply;
              temp.numerator = this->numerator + r.numerator;

              this->numerator /= multiply;
              break;
  }
  return reduce(temp);
}

Rational &Rational:: operator -(Rational r)
{
  int multiply = 0;
  Rational temp;

  if(this->denuminator % r.denuminator == 0)
    multiply = 1;

  if(r.denuminator % this->denuminator == 0)
    multiply = 2;

  switch(multiply){
     case 0: temp.denuminator = this->denuminator * r.denuminator;

             this->numerator *= r.denuminator;
             r.numerator *= this->denuminator;
             temp.numerator = this->numerator - r.numerator;

             this->numerator /= r.denuminator;
             r.numerator /= this->denuminator;
             break;

     case 1:  multiply = this->denuminator / r.denuminator;
              temp.denuminator = this->denuminator;

              r.numerator *= multiply;
              temp.numerator = this->numerator - r.numerator;

              r.numerator /= multiply;
              break;

     case 2:  multiply =  r.denuminator /this->denuminator ;
              temp.denuminator = r.denuminator;

              this->numerator *= multiply;
              temp.numerator = this->numerator - r.numerator;

              this->numerator /= multiply;
              break;

  }
  return reduce(temp);
}

Rational &Rational:: operator *(Rational r)
{
  Rational temp;

  temp. numerator = this->numerator * r.numerator;
  temp.denuminator = this->denuminator * r.denuminator;

  return reduce(temp);
}

Rational &Rational:: operator /(Rational r)
{
  Rational temp;

  temp. numerator = this->numerator * r.denuminator;
  temp.denuminator = this->denuminator * r.numerator;

  return reduce(temp);
}

Rational &Rational::operator = (Rational r)
{

  this->numerator = r.numerator;
  this->denuminator = r.denuminator;

  return *this;
}

int Rational:: operator >(Rational r)
{
  float over1, over2;

  over1 = (float)this->numerator / this->denuminator;
  over2 = (float)r.numerator / r.denuminator;

  if(over1 > over2)
    return 1;

  else
    return 0;
}

int Rational:: operator <(Rational r)
{
  float over1, over2;

  over1 = (float)this->numerator / this->denuminator;
  over2 = (float)r.numerator / r.denuminator;

  if(over1 < over2)
    return 1;

  else
    return 0;
}

int main()
{
   Rational a, b(5), c(6,2), d, e;

   e = a + b * c ;

   cout << "a = " << a <<"b = " << b <<"c = " << c <<"e = " << e ;
   cout << "\nEnter d values in form a/b\n";
   cin >> d;
   cout << "d= " << d << "\n";
   d.printFP();
   cout << (b > c ? "b is larger" : "c is larger");

   getch();
   return 0;

}
