// Program : TestRat
// File    : Tdriver.cpp
// Homework: 5 (Performing arithmetic with fractions)
// Author  : Huseyin ORTAOVALI
// Date    : 23.05.2001

#include <iomanip.h>
#include <iostream.h>
#include <math.h>           // for "sqrt"
#include <stdlib.h>         // for "strtol"

   class Rational
    {
     friend ostream &operator<<(ostream &, const Rational &);
     friend istream &operator>>(istream &, Rational &);

     public:
       Rational( long=0, long=1 );                     // constructor
       const Rational &operator= ( const Rational & ); // overloading operator =
       const Rational &operator+ ( const Rational & ); // overloading operator +
       const Rational &operator+=( const Rational & ); // overloading operator +=
       const Rational &operator- ( const Rational & ); // overloading operator -
       const Rational &operator-=( const Rational & ); // overloading operator -=
       const Rational &operator* ( const Rational & ); // overloading operator *
       const Rational &operator*=( const Rational & ); // overloading operator *=
       const Rational &operator/ ( const Rational & ); // overloading operator /
       const Rational &operator/=( const Rational & ); // overloading operator /=
       bool operator!=( const Rational & ) const;      // overloading operator !=
       bool operator==( const Rational & ) const;      // overloading operator ==
       bool operator< ( const Rational & ) const;      // overloading operator <
       bool operator<=( const Rational & ) const;      // overloading operator <=
       bool operator> ( const Rational & ) const;      // overloading operator >
       bool operator>=( const Rational & ) const;      // overloading operator >=
       void printFP();                                 // printing Floating Point
     private:
       void Reduce();                  // utility function for reducing rationals
       bool is_prime(long);            // utility function for check prime number
       long numerator;                 // numerator of rational number
       long denumerator;               // denumerator of rational number
    };

void main(void)
 {
  Rational a(2), b(7,35), c(3), d(3,6), e;
  char ch;

   cout<<"a="<<a<<"   b="<<b<<"   c="<<c<<"   d="<<d<<"   e="<<e<<endl;
   e=a+b*c;
   cout<<a<<" + "<<b<<" * "<<c<<" = "<<e<<endl;
   e=b*c+a;
   cout<<b<<" * "<<c<<" + "<<a<<" = "<<e<<endl;
   cout<<"e = ";
   e.printFP();
   cout<<endl;
   a=d+e;                                      // Testing +
   cout<<d<<" + "<<e<<" = "<<a<<endl;
   a=d-e;                                      // Testing -
   cout<<d<<" - "<<e<<" = "<<a<<endl;
   cout<<a<<" - "<<d<<" = ";
   a-=d;                                       // Testing -=
   cout<<a<<endl;
   cout<<a<<" + "<<c<<" = ";
   a+=c;                                       // Testing +=
   cout<<a<<endl;
   cout<<"Please enter a rational number in a/b format : ";
   cin>>a;
   cout<<"You have entered (Reduced if necessary) : "<<a<<endl;
   cin>>ch;
 }

   ostream &operator<<(ostream &output, const Rational &number)
    {
      output << number.numerator << "/" << number.denumerator;
      return output;
    }

// overload >>. It gets first two integer as numerator and denumerator
   istream &operator>>(istream &input, Rational &number)
    {
     char st[100], *denptr, *endptr;
     long num=0, den=0;

      input >> setw(100) >> st;
      num = strtol(st, &endptr, 10);
      do
       {
         denptr = endptr+1;
         den = strtol(denptr, &endptr ,10);
       }
      while (!den);
      number.numerator = num;
      number.denumerator = den;
      number.Reduce();
      return input;
    }

   Rational::Rational( long num, long den )            // constructor
    {
      numerator=num;
      denumerator=den;
      Reduce();
    }

// overload =
   const Rational &Rational::operator=( const Rational &operand )
    {
      numerator = operand.numerator;
      denumerator = operand.denumerator;
      return *this;
    }

// overload +
   const Rational &Rational::operator+( const Rational &addend2 )
    {
     Rational *result;

      result->numerator = numerator * addend2.denumerator + denumerator * addend2.numerator;
      result->denumerator = denumerator * addend2.denumerator;
      result->Reduce();
      return *result;
    }

// overload +=
   const Rational &Rational::operator+=( const Rational &addend2 )
    {
      numerator = numerator * addend2.denumerator + denumerator * addend2.numerator;
      denumerator = denumerator * addend2.denumerator;
      Reduce();
      return *this;
    }

// overload -
   const Rational &Rational::operator-( const Rational &subtrahend )
    {
     Rational *result;

      result->numerator = numerator * subtrahend.denumerator - denumerator * subtrahend.numerator;
      result->denumerator = denumerator * subtrahend.denumerator;
      result->Reduce();
      return *result;
    }

// overload -=
   const Rational &Rational::operator-=( const Rational &subtrahend )
    {
      numerator = numerator * subtrahend.denumerator - denumerator * subtrahend.numerator;
      denumerator = denumerator * subtrahend.denumerator;
      Reduce();
      return *this;
    }

// overload *
   const Rational &Rational::operator*( const Rational &multiplier2 )
    {
     Rational *result;

      result->numerator = numerator * multiplier2.numerator;
      result->denumerator = denumerator * multiplier2.denumerator;
      result->Reduce();
      return *result;
    }

// overload *=
   const Rational &Rational::operator*=( const Rational &multiplier2 )
    {
      numerator *= multiplier2.numerator;
      denumerator *= multiplier2.denumerator;
      Reduce();
      return *this;
    }

// overload /
   const Rational &Rational::operator/( const Rational &divisor )
    {
     Rational *result;

      result->numerator = numerator * divisor.denumerator;
      result->denumerator = denumerator * divisor.numerator;
      result->Reduce();
      return *result;
    }

// overload /=
   const Rational &Rational::operator/=( const Rational &divisor )
    {
      numerator *= divisor.denumerator;
      denumerator *= divisor.numerator;
      Reduce();
      return *this;
    }

// overload !=
   bool Rational::operator!=( const Rational &operand ) const
    {
      if ( numerator*operand.denumerator != denumerator*operand.numerator )
         return true;
      else
         return false;
    }

// overload ==
   bool Rational::operator==( const Rational &operand ) const
    {
      if ( numerator*operand.denumerator == denumerator*operand.numerator )
         return true;
      else
         return false;
    }

// overload <
   bool Rational::operator<( const Rational &operand ) const
    {
      if ( numerator*operand.denumerator < denumerator*operand.numerator )
         return true;
      else
         return false;
    }

// overload <=
   bool Rational::operator<=( const Rational &operand ) const
    {
      if ( numerator*operand.denumerator <= denumerator*operand.numerator )
         return true;
      else
         return false;
    }

// overload >
   bool Rational::operator>( const Rational &operand ) const
    {
      if ( numerator*operand.denumerator > denumerator*operand.numerator )
         return true;
      else
         return false;
    }

// overload >=
   bool Rational::operator>=( const Rational &operand ) const
    {
      if ( numerator*operand.denumerator >= denumerator*operand.numerator )
         return true;
      else
         return false;
    }

// Printing to Floating Point
   void Rational::printFP()
    {
      cout<<(float)numerator/denumerator;
    }

// It reduces the rational numbers
   void Rational::Reduce()
    {
     long *smallest, *biggest, checkup_limit, current=2;

      smallest = (numerator<denumerator) ? &numerator : &denumerator;
      biggest = (numerator>denumerator) ? &numerator : &denumerator;
      if (*smallest)
       {
         checkup_limit = (long) sqrt( (*smallest<0) ? -*smallest : *smallest );
         if (is_prime(*smallest))
          {
            if (!(*biggest % *smallest))
             {
               *biggest = *biggest / *smallest;
               *smallest=1;
             }
          }
         else
          {
            do
             {
               if (*smallest % current)
                  current++;
               else
                {
                  if (*biggest % current)
                     current++;
                  else
                   {
                     numerator = numerator / current;
                     denumerator = denumerator / current;
                   }
                }
             }
            while ( current <= checkup_limit);
          }
       }
      if (denumerator<0)
       {
         numerator=-numerator;
         denumerator=-denumerator;
       }
    }

// It checks that if the input is prime. If it is prime, returns true
   bool Rational::is_prime(long prime)
    {
     long checkup_limit, current=2;

      checkup_limit = (long) sqrt( prime<0 ? -prime : prime );
      while ( current <= checkup_limit)
       {
         if (prime % current)
            current++;
         else
            return false;
       }
      return true;
    }

