#include <iostream>
#include <assert.h>
#include <conio.h>
#include <iomanip>

class Rational {

   // Friend functions
   friend ostream &operator<<( ostream &, const Rational & );
   friend istream &operator>>( istream &, Rational & );

   public:
		Rational( int = 1, int = 1 );

      void reduce( const int, const int );
      void printFP( void ) const;

      // Overloading Functions
      const Rational operator+( const Rational & );
      const Rational operator-( const Rational & );
	   const Rational operator*( const Rational & );
      const Rational operator/( const Rational & );
      const Rational &operator=( const Rational & );
		bool operator==( const Rational & ) const;
      bool operator<( const Rational & ) const;
      bool operator>( const Rational & ) const;

   private:
   	int num;
      int denum;

};

Rational::Rational( int Xnum, int Xdenum )
{
	assert( Xdenum != 0 );

   if ( Xnum < 0 )
   	Xnum = -Xnum;

   reduce( Xnum, Xdenum );
}

void Rational::reduce( const int Xnum, const int Xdenum )
{
	if ( Xnum >= Xdenum ) {

   	if ( Xnum % Xdenum == 0 ) {
      	num = Xnum / Xdenum;
         denum = 1;
      }
      else {
      	num = Xnum;
         denum = Xdenum;
      }

   }
   else {

   	if ( Xdenum % Xnum == 0 ) {
      	denum = Xdenum / Xnum;
         num = 1;
      }
      else {
      	num = Xnum;
         denum = Xdenum;
      }

   }

}

void Rational::printFP( void ) const
{
 	float s = ( float )num / denum;

   cout << setiosflags( ios::fixed ) << "\n" << setprecision( 4 ) << s;
}

const Rational Rational::operator+( const Rational &right )
{
	int tNum, tDenum;

   tNum = right.denum * num + denum * right.num;
   tDenum = denum * right.denum;

   Rational x( tNum, tDenum );

   return x;
}

const Rational Rational::operator-( const Rational &right )
{
	int tNum, tDenum;

   tNum = right.denum * num - denum * right.num;
   tDenum = denum * right.denum;

   Rational x( tNum, tDenum );

   return x;
}

const Rational Rational:: operator*( const Rational &right )
{
	int tNum, tDenum;

   tNum = right.num * num;
   tDenum = denum * right.denum;

   Rational x( tNum, tDenum );

   return x;
}

const Rational Rational::operator/( const Rational &right )
{
	int tNum, tDenum;

   tNum = right.denum * num;
   tDenum = denum * right.num;

   Rational x( tNum, tDenum );

   return x;
}

istream &operator>>( istream &input, Rational &a )
{
   int tNum, tDenum;

	input >> tNum;
   input.ignore();
   input >> tDenum;

   a.reduce(tNum, tDenum);

   return input;
}

ostream &operator<<( ostream &output, const Rational &a )
{
	output << a.num << "/" << a.denum;

   return output;
}

const Rational &Rational::operator=( const Rational &right )
{
	this -> num = right.num;
   this -> denum = right.denum;
   return *this;
}

bool Rational::operator==( const Rational &right ) const
{
	if ( this -> num == right.num && this -> denum == right.denum )
   	return true;
   else
   	return false;
}

bool Rational::operator<( const Rational &right ) const
{
	if ( this -> num * right.denum < this -> denum * right.num )
   	return true;
   else
   	return false;
}

bool Rational::operator>( const Rational &right ) const
{
	if ( this -> num * right.denum > this -> denum * right.num )
   	return true;
   else
   	return false;
}


void main( void )
{
	Rational a, b( 5 ), c( 6 , 9 ), d, e;

	e = a + b * c;

	cout << "a=" << a << " b=" << b << " c=" << c << " e=" << e << "\n";
   cout << "Input d? ";
	cin  >> d;
	cout << "d=" << d;
	d.printFP();
   cout << "\nChecking if b > c ";
	cout << "\n" << ( b > c  ? "b is larger" : "c is larger" );

   getch();
}


