//////////////////////////////////////////////////////////////
//															//
//	RATIONAL.CPP											//
//	The member function definitions  of Rational class are	//
//	made in this file.										//
//															//
//////////////////////////////////////////////////////////////
//															//
//	Hakan URHAN												//
//	9721099													//
//	b9721099@hacettepe.edu.tr								//
//	Homework #5												//
//															//
//////////////////////////////////////////////////////////////

#include "rational.h"

ostream & operator << ( ostream &output, Rational &inputRational )
{
	// Print out the rational number in a/b format
	output	<< inputRational.getNumerator() 
			<< "/"
			<< inputRational.getDenominator();
	return output;
}

istream & operator >> ( istream &input, Rational &outputRational )
{
	// Read the rational number in a/b format
	input >> outputRational.numerator;
	input.ignore(); // Ignore '/' character
	input >> outputRational.denominator;

	// Check denominator validity
	if ( outputRational.denominator <= 0 ) {
		cout << "Invalid denominator!" << endl;
		outputRational.denominator = 1;
	}
	
	// Reduce the rational number
	outputRational.reduce();

	return input;
}

// Constructor
Rational::Rational( int numeratorInit, int denominatorInit )
{ 
	// Use setRational function to set values of numerator
	// and denominator to init values
	setRational( numeratorInit, denominatorInit ); 

	// Reduce the rational number
	reduce();
}

Rational Rational::operator +( Rational &sourceRational ) const
{
	int tempNumerator, tempDenominator;
	int sourceNumerator, sourceDenominator;

	sourceNumerator = sourceRational.getNumerator();
	sourceDenominator = sourceRational.getDenominator();

	// Calculate the new numerator and denominator
	tempNumerator = sourceDenominator * numerator + sourceNumerator * denominator;
	tempDenominator = sourceDenominator * denominator;
	
	// Create a new instance of Rational class with new numerator
	// and denominator values, and return it
	return Rational( tempNumerator, tempDenominator );
}

Rational Rational::operator -( Rational &sourceRational ) const
{
	int tempNumerator, tempDenominator;
	int sourceNumerator, sourceDenominator;

	sourceNumerator = sourceRational.getNumerator();
	sourceDenominator = sourceRational.getDenominator();

	// Calculate the new numerator and denominator
	tempNumerator = sourceDenominator * numerator - sourceNumerator * denominator;
	tempDenominator = sourceDenominator * denominator;
	
	// Create a new instance of Rational class with new numerator
	// and denominator values, and return it
	return Rational( tempNumerator, tempDenominator );
}

Rational Rational::operator *( Rational &sourceRational ) const
{
	int tempNumerator, tempDenominator;
	int sourceNumerator, sourceDenominator;

	sourceNumerator = sourceRational.getNumerator();
	sourceDenominator = sourceRational.getDenominator();

	// Calculate the new numerator and denominator
	tempNumerator = numerator * sourceNumerator;
	tempDenominator = sourceDenominator * denominator;
	
	// Create a new instance of Rational class with new numerator
	// and denominator values, and return it
	return Rational( tempNumerator, tempDenominator );
}

Rational Rational::operator /( Rational &sourceRational ) const
{
	int tempNumerator, tempDenominator;
	int sourceNumerator, sourceDenominator;

	sourceNumerator = sourceRational.getNumerator();
	sourceDenominator = sourceRational.getDenominator();

	// Calculate the new numerator and denominator
	tempNumerator = numerator * sourceDenominator;
	tempDenominator = sourceNumerator * denominator;
	
	// Create a new instance of Rational class with new numerator
	// and denominator values, and return it
	return Rational( tempNumerator, tempDenominator );
}

void Rational::setRational( int numeratorSet, int denominatorSet )
{
	// Check the denominator validity and set the values of
	// numerator and denominator values accordinf to given
	// values
	if ( denominatorSet > 0 ) {
		denominator = denominatorSet;
	} else {
		cout << "Invalid denominator!" << endl;
		denominator = 1;
	}
	numerator = numeratorSet;
}

void Rational::printFP()
{
	// Print values of rational number in floating point format
	cout << (float) numerator / (float) denominator << endl;
}

int Rational::operator == ( Rational &rightRational ) const
{
	// Check if the two rational numbers' numerators and
	// denominators are same. If so return 1, if they are
	// different return 0
	if ( ( numerator == rightRational.getNumerator() ) &&
		 ( denominator == rightRational.getDenominator() ) )
		return 1;
	else
		return 0;
}

int Rational::operator < ( Rational &rightRational ) const
{
	double valueThis, valueRight;

	// First calculate the floating point represantation of
	// the rational number and compare them. According to the
	// result give 1 or 0.
	valueThis = (double) numerator / (double) denominator;
	valueRight = (double) rightRational.getNumerator() / (double) rightRational.getDenominator();

	if ( valueThis < valueRight )
		return 1;
	else
		return 0;
}

int Rational::operator > ( Rational &rightRational ) const
{
	double valueThis, valueRight;

	// First calculate the floating point represantation of
	// the rational number and compare them. According to the
	// result give 1 or 0.
	valueThis = (double) numerator / (double) denominator;
	valueRight = (double) rightRational.getNumerator() / (double) rightRational.getDenominator();

	if ( valueThis > valueRight )
		return 1;
	else
		return 0;
}

void Rational::reduce()
{
	int i = 2;
	int minElement = ( numerator < denominator ) ? numerator : denominator;

	// Count up to minimum of numerator and denominator. Divide numerator
	// and denominator with this counter. If the remainders of these 
	// divisions are both 0, then that counter value is a common multiplier
	// of numerator and denominator, so divide both of them to that counter
	// and continue the same process with the new values of numerator and
	// denominator
	while ( i <= denominator ) {
		if ( ( (numerator % i) == 0 ) && ( (denominator % i) == 0 ) ) {
			numerator /= i;
			denominator /= i;
		}
		i++;
	}
}
